//-------------------------------------------------------------------
//	File:		YourCode.cpp
//	Comments:	Write your visual effect here.
//-------------------------------------------------------------------
#include "std.h"      // Include the standard header.
#include "RTexture.h" // We use textures, just for kicks.
#include "imgLoadBMP.h" // Image loader handler for RTexture/RImage.

// Personal headers

#define MAIN
#define MAIN_GLEXT

#include "HDef.h"
#include "HScene.h"
#include "glExt.h"
#include "glExt_compiled_vertex_array.h"
#include "MouseMan.h"


// This header binds this code with the system framework code.
#include "OGLCMainHeader.h"

// Return the application info.
// Functions implemented by the author in YourCode.cpp
bool appPreInit();  // Initialize before OpenGL has been initialized.
bool appInit();     // Initialize after OpenGL has been initialized.
bool appRender();   // Update/Render a frame.
void appShutdown(); // Take down any textures, timers, sounds, etc...

// Return a null terminated string for each field.
char *appGetAuthor()        {return "Hufo / N.A.A.";}
char *appGetName()          {return "Tank Rocket";}
char *appGetDescription()   {return
"This is a little tank game with only one weapon:\n"
"ROCKET! You have seven random-controlled enemy to echange\n"
"rocket with. Your tank is controlled by cursor keys, the\n"
"turret is mouse controlled.\n"
"Have FUN!";
}

// Title updated every half second on Win32 systems.
//char *appGetTitleBar()      {return "Crystal Reflection by Hufo / N.A.A. [fps: +inf :-)]";}
char *appGetTitleBar()
{
	return HText("Tank Rocket by Hufo / N.A.A. [fps: +inf :-)] np=%d",PartSyst.np);
}

extern HWND g_hwnd;
extern HINSTANCE g_hinst;

//----------------------------------
// Timer info is stored in:
// sys_Time - counts in milliseconds. (1/1000th of a second)
// Very useful for keeping animated stuff in sync on different computers.
//---
// Current window dimensions are stored in:
// sys_glWidth, sys_glHeight.
/*
// Use the demonstration texture.
RTexture texExample;
*/

// May be useful to some people.
bool appPreInit() // Called before OpenGL has been initialized.
{
	Test(HTank::Load());
	Test(HMissile::Load());
	Test(HParticule::Load());
	PartSyst.Alloc(5000);

	//Test(oArena.LoadLWO("data/arena1.lwo"));
	Test(oArena.LoadLWO("data/sol1.lwo"));
	Test(oSky.LoadLWO("data/sky.lwo"));

	oArena.CalcBBox(&bbArena);

	//bbArena=bbSol;
	bbArena.b.y+=50.0f;

	HTank::SetArena(&bbArena);
	HMissile::SetArena(&bbArena);
	HMissile::SetPartSyst(&PartSyst);

	PlayerTank=&(TTank[0]);

	for (int i=0;i<NTANK;i++)
	{
		TTank[i].Pos.x=bbArena.a.x+(bbArena.b.x-bbArena.a.x)*rnd();
		TTank[i].Pos.z=bbArena.a.z+(bbArena.b.z-bbArena.a.z)*rnd();
		TTank[i].Angle=(float)RAD2DEG(atan2(-TTank[i].Pos.x+(bbArena.b.x+bbArena.a.x)*0.5f
		                                   ,-TTank[i].Pos.z+(bbArena.b.z+bbArena.a.z)*0.5f));

		if (i)
		{
			TTankAI[i]=new HTankAI_Random(&(TTank[i]));
		}
		else
			TTankAI[i]=NULL;

	}

  return true;
}

// Startup Stuff.
bool appInit()	// Called right after the window is created, and OpenGL is initialized.
{

	Test(MouseManInit(g_hinst,g_hwnd));

	glGetExtension(&glExt_compiled_vertex_array);

	oArena.LoadTexture();
	oSky.LoadTexture();
	HTank::LoadTexture();
	HMissile::LoadTexture();
	HParticule::LoadTexture();

  // Create a simple camera/projection matrix.
  glMatrixMode (GL_PROJECTION);
  glLoadIdentity ();
  gluPerspective(90, (GLdouble)sys_glWidth/(GLdouble)sys_glHeight, 1.0, 1000.0);

  // And simple model matrix.
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  //glTranslatef( 0,-2,-5 ); // Move back a touch.

  // Clear the screen for the first time...
  glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	glShadeModel(GL_SMOOTH);
	glEnable(GL_DEPTH_TEST);
	//glDisable(GL_DEPTH_TEST);

	//glFrontFace(GL_CW);

	// If everything went well, then return true.
	// Returning false will cause the program to quit right away.
	return true;
}

bool KeyLeft = false;
bool KeyRight = false;
bool KeyUp = false;
bool KeyDown = false;

void appKeyDown(int k)
{
	switch(k)
	{
		case VK_LEFT: KeyLeft = true; break;
		case VK_RIGHT: KeyRight = true; break;
		case VK_UP: KeyUp = true; break;
		case VK_DOWN: KeyDown = true; break;
		//case VK_SPACE: InitFire(); break;
	}
}

void appKeyUp(int k)
{
	switch(k)
	{
		case VK_LEFT: KeyLeft = false; break;
		case VK_RIGHT: KeyRight = false; break;
		case VK_UP: KeyUp = false; break;
		case VK_DOWN: KeyDown = false; break;
	}
}

void appKeyChar(int k)
{
}

// Draw all the scene related stuff.
bool appRender()
{
  // We like to use floating point seconds, just for kicks.

	static bool first=true;
	static double lastTime;
  double fTime = sys_Time/1000.f;
	if (first)
	{
		lastTime=fTime;
	}
	int i;

	float t=(float)fTime;
	float dt=(float)(fTime-lastTime);

	float da=0;

	if (KeyLeft) da+=90;
	if (KeyRight) da-=90;

	PlayerTank->RotateTank(da);

	float dp=0;

	if (KeyUp) dp+=10.0f;
	if (KeyDown) dp-=5.0f;

	PlayerTank->DeplTank(dp);

	//PlayerTank->TurretAngle=fTime*90.0f;
	static int lastbt=0;
	int dx,dy,bt;
	if (!first)
	{
		MouseManGetDepl(&dx,&dy,&bt);
		PlayerTank->RotateTurret(-(float)dx*0.5f);
		PlayerTank->RotateGun((float)dy*0.5f);
		if ((bt&MOUSE_BTG) && !(lastbt&MOUSE_BTG))
		{
			NewMissile();
		}
		lastbt=bt;
	}

	for(i=0;i<NTANK;i++)
	{
		if (TTankAI[i]!=NULL)
			TTankAI[i]->Update(t,dt);
		TTank[i].Update(t,dt);
	}
	UpdateMissile(t,dt);
	PartSyst.Update(t,dt);

  // Clear the screen
  glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

  // And simple model matrix.
  glMatrixMode(GL_MODELVIEW);

  //glLoadIdentity();
  //glTranslatef( 0,-2,-10 ); // Move back a touch.
	//glRotatef((float)fTime*(6*PI),0,1,0);

	HPoint3D pCam(0,3,-6);
	//HPoint3D pCam(0,20,-60);
	PlayerTank->SetCam(pCam);

	HColor4f ambient(1.0f,1.0f,1.0f);
	HColor4f clight(1.0f,1.0f,1.0f);
	float light0[4]={-1.0f,0.1f,-1.0f,0.0f};
	glLightfv(GL_LIGHT0,GL_AMBIENT,ambient);
	glLightfv(GL_LIGHT0,GL_DIFFUSE,clight);
	glLightfv(GL_LIGHT0,GL_POSITION,light0);
	glEnable(GL_LIGHT0);


	float light1[4]={1.0f,-0.1f,0.5f,0.0f};
	glLightfv(GL_LIGHT1,GL_DIFFUSE,clight);
	glLightfv(GL_LIGHT1,GL_POSITION,light1);
	glEnable(GL_LIGHT1);

	glDisable(GL_DEPTH_TEST);

	oSky.Aff();
	oArena.Aff();

	glEnable(GL_DEPTH_TEST);

	for(i=0;i<NTANK;i++)
	{
		TTank[i].Aff();
	}

	AffMissile();
	
	{
		float cam_angle=DEG2RAD(PlayerTank->GetCamAngle());

		HPoint3D dx((float)cos(cam_angle),0,-(float)sin(cam_angle));
		HPoint3D dy(0,1,0);

		PartSyst.Aff(dx,dy);
	}

	lastTime=fTime; first=false;
  return true;
}

void appShutdown() // Called right after the window is destroyed.
{

	MouseManQuit();

	FreeMissile();

	//HTank::oTank.Free();
	//HTank::oTurret.Free();
	//HTank::oGun.Free();

/*
	HPolygon2D::FreeTable();
*/
  // We're finished with the texture.
  //texExample.Toast();
}
